/**
 * -------------------------------------------------------------------------------------------
 * Copyright (c) Microsoft Corporation.  All Rights Reserved.  Licensed under the MIT License.
 * See License in the project root for license information.
 * -------------------------------------------------------------------------------------------
 */
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { LitElement } from 'lit';
import { state } from 'lit/decorators.js';
import { ProviderState } from '../providers/IProvider';
import { Providers } from '../providers/Providers';
import { LocalizationHelper } from '../utils/LocalizationHelper';
import { PACKAGE_VERSION } from '../utils/version';
/**
 * Defines media query based on component width
 *
 * @export
 * @enum {string}
 */
export var ComponentMediaQuery;
(function (ComponentMediaQuery) {
    /**
     * devices with width < 768
     */
    ComponentMediaQuery["mobile"] = "";
    /**
     * devices with width < 1200
     */
    ComponentMediaQuery["tablet"] = "tablet";
    /**
     * devices with width > 1200
     */
    ComponentMediaQuery["desktop"] = "desktop";
})(ComponentMediaQuery || (ComponentMediaQuery = {}));
/**
 * BaseComponent extends LitElement adding mgt specific features to all components
 *
 * @export  MgtBaseComponent
 * @abstract
 * @class MgtBaseComponent
 * @extends {LitElement}
 * @deprecated Use MgtBaseTaskComponent instead
 */
export class MgtBaseComponent extends LitElement {
    /**
     * Exposes the semver of the library the component is part of
     *
     * @readonly
     * @static
     * @memberof MgtBaseComponent
     */
    static get packageVersion() {
        return PACKAGE_VERSION;
    }
    /**
     * Gets the ComponentMediaQuery of the component
     *
     * @readonly
     * @type {MgtElement.ComponentMediaQuery}
     * @memberof MgtBaseComponent
     */
    get mediaQuery() {
        if (this.offsetWidth < 768) {
            return ComponentMediaQuery.mobile;
        }
        else if (this.offsetWidth < 1200) {
            return ComponentMediaQuery.tablet;
        }
        else {
            return ComponentMediaQuery.desktop;
        }
    }
    /**
     * A flag to check if the component is loading data state.
     *
     * @protected
     * @memberof MgtBaseComponent
     */
    get isLoadingState() {
        return this._isLoadingState;
    }
    /**
     * A flag to check if the component has updated once.
     *
     * @readonly
     * @protected
     * @type {boolean}
     * @memberof MgtBaseComponent
     */
    get isFirstUpdated() {
        return this._isFirstUpdated;
    }
    /**
     * returns component strings
     *
     * @readonly
     * @protected
     * @memberof MgtBaseComponent
     */
    get strings() {
        return {};
    }
    constructor() {
        super();
        this.providerState = ProviderState.Loading;
        /**
         * Gets or sets the direction of the component
         *
         * @protected
         * @memberof MgtBaseComponent
         */
        this.direction = 'ltr';
        /**
         * determines if login component is in loading state
         *
         * @type {boolean}
         */
        this._isLoadingState = false;
        this._isFirstUpdated = false;
        this.setLoadingState = (value) => {
            if (this._isLoadingState === value) {
                return;
            }
            this._isLoadingState = value;
            this.requestUpdate('isLoadingState');
        };
        this.handleProviderUpdates = () => {
            var _a, _b;
            this.providerState = (_b = (_a = Providers.globalProvider) === null || _a === void 0 ? void 0 : _a.state) !== null && _b !== void 0 ? _b : ProviderState.Loading;
            void this.requestStateUpdate();
        };
        this.handleActiveAccountUpdates = () => {
            this.clearState();
            void this.requestStateUpdate();
        };
        this.handleLocalizationChanged = () => {
            LocalizationHelper.updateStringsForTag(this.tagName, this.strings);
            this.requestUpdate();
        };
        this.handleDirectionChanged = () => {
            this.direction = LocalizationHelper.getDocumentDirection();
        };
        this.handleDirectionChanged();
        this.handleLocalizationChanged();
    }
    /**
     * Invoked each time the custom element is appended into a document-connected element
     *
     * @memberof MgtBaseComponent
     */
    connectedCallback() {
        super.connectedCallback();
        LocalizationHelper.onStringsUpdated(this.handleLocalizationChanged);
        LocalizationHelper.onDirectionUpdated(this.handleDirectionChanged);
    }
    /**
     * Invoked each time the custom element is removed from a document-connected element
     *
     * @memberof MgtBaseComponent
     */
    disconnectedCallback() {
        super.disconnectedCallback();
        LocalizationHelper.removeOnStringsUpdated(this.handleLocalizationChanged);
        LocalizationHelper.removeOnDirectionUpdated(this.handleDirectionChanged);
        Providers.removeProviderUpdatedListener(this.handleProviderUpdates);
        Providers.removeActiveAccountChangedListener(this.handleActiveAccountUpdates);
    }
    /**
     * Invoked when the element is first updated. Implement to perform one time
     * work on the element after update.
     *
     * Setting properties inside this method will trigger the element to update
     * again after this update cycle completes.
     *
     * @param _changedProperties Map of changed properties with old values
     */
    firstUpdated(changedProperties) {
        super.firstUpdated(changedProperties);
        this._isFirstUpdated = true;
        Providers.onProviderUpdated(this.handleProviderUpdates);
        Providers.onActiveAccountChanged(this.handleActiveAccountUpdates);
        void this.requestStateUpdate();
    }
    /**
     * load state into the component.
     * Override this function to provide additional loading logic.
     */
    loadState() {
        return Promise.resolve();
    }
    /**
     * Used to clear state in inherited components
     */
    clearState() {
        // no-op
    }
    /**
     * helps facilitate creation of events across components
     *
     * @protected
     * @param {string} eventName
     * @param {*} [detail]
     * @param {boolean} [bubbles=false]
     * @param {boolean} [cancelable=false]
     * @param {boolean} [composed=false]
     * @return {*}  {boolean}
     * @memberof MgtBaseComponent
     */
    fireCustomEvent(eventName, detail, bubbles = false, cancelable = false, composed = false) {
        const event = new CustomEvent(eventName, {
            bubbles,
            cancelable,
            composed,
            // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment
            detail
        });
        return this.dispatchEvent(event);
    }
    /**
     * Invoked whenever the element is updated. Implement to perform
     * post-updating tasks via DOM APIs, for example, focusing an element.
     *
     * Setting properties inside this method will trigger the element to update
     * again after this update cycle completes.
     *
     * * @param changedProperties Map of changed properties with old values
     */
    updated(changedProperties) {
        super.updated(changedProperties);
        const event = new CustomEvent('updated', {
            bubbles: true,
            cancelable: true
        });
        this.dispatchEvent(event);
    }
    /**
     * Request to reload the state.
     * Use reload instead of load to ensure loading events are fired.
     *
     * @protected
     * @memberof MgtBaseComponent
     */
    requestStateUpdate(force = false) {
        return __awaiter(this, void 0, void 0, function* () {
            // the component is still bootstraping - wait until first updated
            if (!this._isFirstUpdated) {
                return;
            }
            // Wait for the current load promise to complete (unless forced).
            if (this.isLoadingState && !force) {
                yield this._currentLoadStatePromise;
            }
            const provider = Providers.globalProvider;
            if (!provider) {
                return Promise.resolve();
            }
            if (provider.state === ProviderState.SignedOut) {
                // Signed out, clear the component state
                this.clearState();
                return;
            }
            else if (provider.state === ProviderState.Loading) {
                // The provider state is indeterminate. Do nothing.
                return Promise.resolve();
            }
            else {
                // Signed in, load the internal component state
                // eslint-disable-next-line @typescript-eslint/no-misused-promises, no-async-promise-executor
                const loadStatePromise = new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                    try {
                        this.setLoadingState(true);
                        this.fireCustomEvent('loadingInitiated');
                        yield this.loadState();
                        this.setLoadingState(false);
                        this.fireCustomEvent('loadingCompleted');
                        resolve();
                    }
                    catch (e) {
                        // Loading failed. Clear any partially set data.
                        this.clearState();
                        this.setLoadingState(false);
                        this.fireCustomEvent('loadingFailed');
                        reject(e);
                    }
                    // Return the load state promise.
                    // If loading + forced, chain the promises.
                    // This is to account for the lack of a cancellation token concept.
                    // eslint-disable-next-line @typescript-eslint/no-unsafe-return, @typescript-eslint/no-unsafe-assignment
                    return (this._currentLoadStatePromise =
                        // eslint-disable-next-line @typescript-eslint/no-misused-promises
                        this.isLoadingState && !!this._currentLoadStatePromise && force
                            ? // eslint-disable-next-line @typescript-eslint/no-unsafe-return
                                this._currentLoadStatePromise.then(() => loadStatePromise)
                            : loadStatePromise);
                }));
            }
        });
    }
}
__decorate([
    state(),
    __metadata("design:type", Number)
], MgtBaseComponent.prototype, "providerState", void 0);
__decorate([
    state(),
    __metadata("design:type", String)
], MgtBaseComponent.prototype, "direction", void 0);
//# sourceMappingURL=baseComponent.js.map